@@ -8,7 +8,7 @@ use std::sync::Arc;
 use vrp_cli::extensions::solve::config::create_builder_from_config_file;
 use vrp_cli::{get_errors_serialized, get_locations_serialized};
 use vrp_core::models::{Problem, Solution};
-use vrp_core::solver::Builder;
+use vrp_core::solver::{Builder, Metrics, Telemetry, TelemetryMode};
 use vrp_pragmatic::format::problem::{deserialize_problem, PragmaticProblem};
 use vrp_pragmatic::format::solution::PragmaticSolution;
 use vrp_scientific::common::read_init_solution;
@@ -27,6 +27,7 @@ const INIT_SOLUTION_ARG_NAME: &str = "init-solution";
 const OUT_RESULT_ARG_NAME: &str = "out-result";
 const GET_LOCATIONS_ARG_NAME: &str = "get-locations";
 const CONFIG_ARG_NAME: &str = "config";
+const LOG_ARG_NAME: &str = "log";
 
 #[allow(clippy::type_complexity)]
 struct ProblemReader(pub Box<dyn Fn(File, Option<Vec<File>>) -> Result<Problem, String>>);
@@ -36,7 +37,13 @@ struct InitSolutionReader(pub Box<dyn Fn(File, Arc<Problem>) -> Option<Solution>
 #[allow(clippy::type_complexity)]
 struct SolutionWriter(
     pub  Box<
-        dyn Fn(&Problem, Solution, BufWriter<Box<dyn Write>>, Option<BufWriter<Box<dyn Write>>>) -> Result<(), String>,
+        dyn Fn(
+            &Problem,
+            Solution,
+            Option<Metrics>,
+            BufWriter<Box<dyn Write>>,
+            Option<BufWriter<Box<dyn Write>>>,
+        ) -> Result<(), String>,
     >,
 );
 
@@ -53,7 +60,7 @@ fn get_formats<'a>() -> HashMap<&'a str, (ProblemReader, InitSolutionReader, Sol
                     BufReader::new(problem).read_solomon()
                 })),
                 InitSolutionReader(Box::new(|file, problem| read_init_solution(BufReader::new(file), problem).ok())),
-                SolutionWriter(Box::new(|_, solution, writer, _| solution.write_solomon(writer))),
+                SolutionWriter(Box::new(|_, solution, _, writer, _| solution.write_solomon(writer))),
                 LocationWriter(Box::new(|_, _| unimplemented!())),
             ),
         ),
@@ -65,7 +72,7 @@ fn get_formats<'a>() -> HashMap<&'a str, (ProblemReader, InitSolutionReader, Sol
                     BufReader::new(problem).read_lilim()
                 })),
                 InitSolutionReader(Box::new(|_file, _problem| None)),
-                SolutionWriter(Box::new(|_, solution, writer, _| solution.write_lilim(writer))),
+                SolutionWriter(Box::new(|_, solution, _, writer, _| solution.write_lilim(writer))),
                 LocationWriter(Box::new(|_, _| unimplemented!())),
             ),
         ),
@@ -83,10 +90,16 @@ fn get_formats<'a>() -> HashMap<&'a str, (ProblemReader, InitSolutionReader, Sol
                     .map_err(|errors| errors.iter().map(|err| err.to_string()).collect::<Vec<_>>().join("\t\n"))
                 })),
                 InitSolutionReader(Box::new(|_file, _problem| None)),
-                SolutionWriter(Box::new(|problem, solution, default_writer, geojson_writer| {
+                SolutionWriter(Box::new(|problem, solution, metrics, default_writer, geojson_writer| {
                     geojson_writer
                         .map_or(Ok(()), |geojson_writer| solution.write_geo_json(problem, geojson_writer))
-                        .and_then(|_| solution.write_pragmatic_json(problem, default_writer))
+                        .and_then(|_| {
+                            if let Some(metrics) = metrics {
+                                (solution, metrics).write_pragmatic_json(problem, default_writer)
+                            } else {
+                                solution.write_pragmatic_json(problem, default_writer)
+                            }
+                        })
                 })),
                 LocationWriter(Box::new(|problem, writer| {
                     let mut writer = writer;
@@ -185,6 +198,13 @@ pub fn get_solve_app<'a, 'b>() -> App<'a, 'b> {
                 .required(false)
                 .takes_value(true),
         )
+        .arg(
+            Arg::with_name(LOG_ARG_NAME)
+                .help("Specifies whether default logging is enabled")
+                .long(LOG_ARG_NAME)
+                .required(false)
+                .takes_value(false),
+        )
 }
 
 /// Runs solver commands.
@@ -199,6 +219,11 @@ pub fn run_solve(matches: &ArgMatches) {
     // optional
     let max_generations = parse_int_value::<usize>(matches, GENERATIONS_ARG_NAME, "max generations");
     let max_time = parse_int_value::<usize>(matches, TIME_ARG_NAME, "max time");
+    let telemetry = Telemetry::new(if matches.is_present(LOG_ARG_NAME) {
+        TelemetryMode::OnlyLogging { logger: Arc::new(|msg| println!("{}", msg)), log_best: 100, log_population: 1000 }
+    } else {
+        TelemetryMode::None
+    });
 
     let cost_variation = matches.value_of(COST_VARIATION_ARG_NAME).map(|arg| {
         if let [sample, threshold] =
@@ -247,9 +272,10 @@ pub fn run_solve(matches: &ArgMatches) {
                                 .with_max_generations(max_generations)
                                 .with_max_time(max_time)
                                 .with_cost_variation(cost_variation)
+                                .with_telemetry(telemetry)
                         };
 
-                        let (solution, _) = builder
+                        let (solution, _, metrics) = builder
                             .with_solutions(solution.map_or_else(|| vec![], |s| vec![Arc::new(s)]))
                             .build()
                             .and_then(|solver| solver.solve())
@@ -258,7 +284,7 @@ pub fn run_solve(matches: &ArgMatches) {
                                 process::exit(1);
                             });
 
-                        solution_writer.0(&problem, solution, out_buffer, geo_buffer).unwrap()
+                        solution_writer.0(&problem, solution, metrics, out_buffer, geo_buffer).unwrap()
                     }
                     Err(error) => {
                         eprintln!("cannot read {} problem from '{}': '{}'", problem_format, problem_path, error);
